from django.db import models
from reqman.apps.account.models import User

from pygments.formatters.html import HtmlFormatter
from pygments import highlight

from reqman.apps.reqtool.rest_api.services.parse_ontologies import *

findsubclass()

MAIN_CHOICES = ( ("",""),   ("M1", "M1 : system/function shall [not] set [<quantifier>] item [to stateValue]"), 
                            ("M2", "M2 : system/function shall [not] set state to stateValue"), 
                            ("M3", "M3 : system/function shall [not] send [<quantifier>] item"),
                            ("M4", "M4 : system/function shall [not] receive [<quantifier>] item"),
                            ("M5", "M5 : system/function shall [not] ingest flow"),
                            ("M6", "M6 : system/function shall [not] emit flow"),
                            ("M7", "M7 : system shall [not] perform function"),
                            ("M8", "M8 : function shall [not] invoke function"),
                            ("M9", "M9 : system shall [not] present interface [to system]"),
                            ("M10", "M10 : system/interface shall [not] transfer flow/item"),
                            ("M11", "M11 : system shall [not] interact with system [using <connection>] [as dened in <text>]"),
                            ("M12", "M12 : system shall [not] have state state [with values stateSet]"),
                            ("M13", "M13 : system state stateValue shall [not] have substate state [with values stateSet]"),
                            ("M14", "M14 : system item shall [not] take values from stateSet"),
                            ("M15", "M15 : system item shall [not] be composed from item"),
                            ("M16", "M16 : system shall [not] contain system")
                        )

#to be updated with domains from DSO
SYSTEM_CHOICES = get_instances("SAO#System")
FUNCTION_CHOICES = get_instances("SAO#Function")
SHALL_CHOICES = ( ("shall", "SHALL"), ("shall not", "SHALL NOT"))
QUANTIFIER_CHOICES = ( ("none","NONE"), ("all", "ALL"), ("only", "ONLY"),
                                ("more than", "MORE THAN"), ("less than", "LESS THAN"), ("exactly","EXACTLY"), 
                                ("at least", "AT LEAST"), ("at most", "AT MOST")
                     )
NUMBER_UNITS_CHOICES = (("",""), ("meters","METERS"), ("kilometers","KILOMETERS"), ("volt","VOLT"))
ITEM_CHOICES = get_instances("SAO#Item")
STATE_VALUE_CHOICES =get_instances("SAO#StateValue")
STATE_CHOICES = get_instances("SAO#State")#tuple( [ x for x in find_extra_keywords(":State", 2) if x not in STATE_VALUE_CHOICES ])
FLOW_CHOICES = get_instances("SAO#Flow")
INTERFACE_CHOICES = get_instances("SAO#Interface")
CONNECTION_CHOICES = get_instances("SAO#Connection")
VERB_CHOICES = (("set","set"),("send","send"),("receive","receive"),("ingest","ingest"),("emit","emit"),("perform","perform"),("invoke","invoke"),
                ("present","present"),("transfer","transfer"),("interact with","interact with"),("have state","have state"),("have substate","have substate"),
                ("take values from","take values from"),("be composed","be composed"),("contain","contain"))
SYSTEM_INTERFACE_CHOICES = SYSTEM_CHOICES + INTERFACE_CHOICES
SYSTEM_OR_FUNCTION_CHOICES = SYSTEM_CHOICES + FUNCTION_CHOICES
FLOW_ITEM_CHOICES = FLOW_CHOICES + ITEM_CHOICES
SYSTEM_FUNCTION_INTERFACE_CHOICES = SYSTEM_CHOICES + FUNCTION_CHOICES + ITEM_CHOICES
FLOW_FUNCTION_INTERFACE_ITEM_SYSTEM_STATE_CHOICES = FLOW_CHOICES + FUNCTION_CHOICES + INTERFACE_CHOICES + ITEM_CHOICES + SYSTEM_CHOICES + STATE_CHOICES 
ITEM_FLOW_FUNCTION = ITEM_CHOICES + FLOW_CHOICES + FUNCTION_CHOICES


def create_main(instance):
    '''Create the Main Model for the Boilerplate'''
    try :
        main = Main.objects.get(boilerplate_of_main = instance, main_owner = instance.owner)
        if instance.has_main:
            return
        main = Main.objects.get(boilerplate_of_main = instance, main_owner = instance.owner)
        main.delete()
    except Main.DoesNotExist:
        if instance.has_main:
            main = Main(boilerplate_of_main = instance, main_owner = instance.owner)
            main.save()


class Main(models.Model):
    boilerplate_of_main = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate', on_delete=models.CASCADE)
    main_owner = models.ForeignKey(User, related_name='main_owner', on_delete=models.CASCADE)
    #Subject
    sys_fun_inter = models.CharField(choices=SYSTEM_FUNCTION_INTERFACE_CHOICES, max_length=100)
    #Between Subject - Verb
    state_before_verb = models.CharField(choices=STATE_CHOICES, max_length=100, blank=True)
    statevalue_before_verb = models.CharField(choices=STATE_VALUE_CHOICES, max_length=100, blank=True)
    item_before_verb = models.CharField(choices=ITEM_CHOICES, max_length=100, blank=True )
    #Verb
    shall = models.CharField(choices=SHALL_CHOICES, max_length=100)
    verb = models.CharField(choices=VERB_CHOICES, max_length=100)
    #Between Verb - Object
    quantifier = models.CharField(choices=QUANTIFIER_CHOICES, max_length=100, blank=True)
    numerical = models.IntegerField(default=0, blank=True) 
    mumerical_units = models.CharField(choices=NUMBER_UNITS_CHOICES, max_length=100)
    item = models.CharField(choices=ITEM_CHOICES, max_length=100, blank=True)
    state = models.CharField(choices=STATE_CHOICES, max_length=100, blank=True)
    statevalue = models.CharField(choices=STATE_VALUE_CHOICES, max_length=100, blank=True)
    #Οbject
    #add stateSet
    flow_function_interface_item_system_state = models.CharField(choices=FLOW_FUNCTION_INTERFACE_ITEM_SYSTEM_STATE_CHOICES, max_length=100, blank=True)
    #last definitions
    connection = models.CharField(choices=CONNECTION_CHOICES, max_length=100, blank=True)
    #ID [M1-M16]
    main_choices = models.CharField(choices=MAIN_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_main']

